/* ------------------------------------------------------------

Problem Description
-------------------

This is a problem of building five houses. The masonry, roofing,
painting, etc. must be scheduled.  Some tasks must necessarily take
place before others and these requirements are expressed through
precedence constraints.

There are two workers and each task requires a specific worker.  The
worker has a calendar of days off that must be taken into account. The
objective is to minimize the overall completion date.

------------------------------------------------------------ */
 
using CP;

int NbHouses = ...; 
range Houses = 1..NbHouses;

{string} WorkerNames = ...;  
{string} TaskNames   = ...;

int    Duration [t in TaskNames] = ...;
string Worker   [t in TaskNames] = ...;

tuple Precedence {
  string doBefore;
  string doAfter;
};

{Precedence} Precedences = ...;

tuple Break {
  int s;
  int e;
};

{Break} Breaks[WorkerNames] = ...; 

// Set of break steps
tuple Step {
  int v;
  key int x;
};
sorted {Step} Steps[w in WorkerNames] = 
   { <100, b.s> | b in Breaks[w] } union 
   { <0, b.e>   | b in Breaks[w] };
   
stepFunction Calendar[w in WorkerNames] = 
  stepwise (s in Steps[w]) { s.v -> s.x; 100 };

dvar interval itvs[h in Houses, t in TaskNames] 
  size      Duration[t]
  intensity Calendar[Worker[t]];

execute {
  cp.param.FailLimit = 10000;
}

minimize max(h in Houses) endOf(itvs[h]["moving"]);

subject to {
  forall(h in Houses) {
    forall(p in Precedences)
     ct1: endBeforeStart(itvs[h][p.doBefore], itvs[h][p.doAfter]);
    forall(t in TaskNames)  
     {
      	ct2: forbidStart(itvs[h][t], Calendar[Worker[t]]);
       	ct3: forbidEnd  (itvs[h][t], Calendar[Worker[t]]);
    }
  }
  forall(w in WorkerNames)
    ct4: noOverlap( all(h in Houses, t in TaskNames: Worker[t]==w) itvs[h][t]);
}
